/* 
*
* Copyright (C) 2007,2010 Wei Lian <lianwei3 at gmail.com>
*
* This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "header.h"
#include "pageplay.h"
#include "pagechannel.h"
#include "channel.h"
#include "mainwindow.h"
#include "mylistitem.h"
#include "config.h"
#include "mplayerfork.h"
#include "vlcfork.h"
#include "mylabel.h"
#include "menubar.h"
#include "tabwidget.h"
#include "utils.h"
//Added by qt3to4:
#include <QMouseEvent>
Channel::Channel (QWidget * parent, const char *name, Qt::WFlags f):
Q3ListView (parent, name, f),
pagechannel (static_cast < PageChannel * >(parent))
{
  //listview
  addColumn (tr ("Name"));
/*  addColumn(tr("Status"));
    addColumn(tr("Type"));
    addColumn(tr("Kbps"));
    addColumn(tr("Users"));
    addColumn(tr("Vists"));
    addColumn(tr("QualityS"));
    addColumn(tr("QualityC"));
    addColumn(tr("StarFrom"));
    addColumn(tr("URL"));
    addColumn(tr("Description"));*/
  setRootIsDecorated (TRUE);

  connect (this, SIGNAL (doubleClicked (Q3ListViewItem *)), this, SLOT (onListItemClicked (Q3ListViewItem *)));
  connect (this, SIGNAL (returnPressed (Q3ListViewItem *)), this, SLOT (onListItemClicked (Q3ListViewItem *)));
  connect (this, SIGNAL (rightButtonClicked (Q3ListViewItem *, const QPoint &, int)), this, SLOT (onRightButtonClicked (Q3ListViewItem *, const QPoint &, int)));
  connect (this, SIGNAL (onItem (Q3ListViewItem *)), this, SLOT (onMouseMoved (Q3ListViewItem *)));

  //popup
  popup = new MyLabel (this);
  //
  //  fetchingchannel = FALSE;
  //buffer
  bufchannel = new QBuffer ();
  ///open bufchannel
  if (!bufchannel->open (IO_ReadWrite))
    {
      perror ("cannot create the buffer");
      return;
    }
  ////
  wget = new Q3Process (this);
  wget->setCommunication (Q3Process::Stdout);
  connect (wget, SIGNAL (processExited ()), this, SLOT (onWgetExit ()));
  connect (wget, SIGNAL (readyReadStdout ()), this, SLOT (wgetStdout ()));

}

Channel::~Channel ()
{
  bufchannel->close ();
  delete bufchannel;
}

void
Channel::onButtonChannelToggled (bool t)
{
  if (t)
    {
      ///reset buffer pointer to 0
      bufchannel->reset ();
      ///launch wget
      wget->clearArguments ();
      wget->addArgument ("wget");
      wget->addArgument (static_cast < MainWindow * >(qApp->mainWidget ())->vboxconfig->editchannelurl->text ());
      wget->addArgument ("-O");
      wget->addArgument ("-");
      wget->start ();
    }
  else
    {
      ///terminate wget
      char str[256];
      strcpy (str, wget->arguments ().join (" ").ascii ());
      killProcess (str);	//, wget->processIdentifier (),
//                 wget->processIdentifier () + 10);
      ///reset color for button channel
      pagechannel->buttonchannel->setPaletteForegroundColor (Qt::black);
    }
}


void
Channel::wgetStdout ()
{
  bufchannel->writeBlock (wget->readStdout ());
  int done = bufchannel->at ();
  ///use button channel forground color to illuminate progress
  int period = 178805;
  int num = (done / period) % 2 ? 255 - (256 * done / period) % 256 : (256 * done / period) % 256;
  ///
  pagechannel->buttonchannel->setPaletteForegroundColor (QColor (0, num, 0));

}


void
Channel::onWgetExit ()
{
  ///reset button channel and reset its color
  pagechannel->buttonchannel->setOn (false);

  ///toggle channel sorting
  toggleChannelSort ();
  ///

  ///reset buffer pointer to 0
  bufchannel->reset ();
  QDomDocument doc;
  doc.setContent (bufchannel);
  ////
  Q3ListViewItem *groupitem = 0;
  MyListItem *item = 0;

  updateOpenState ();
  clear ();

  for (int i = 0; i < N_COLUMNS; i++)
    Max[i] = 1;
  static const char node_name[][20] = { "name", "status", "stream_type", "kbps", "user_count",
    "visit_count", "qs", "qc", "start_from", "sop_address",
    "description"
  };


  ///group
  for (QDomNode group = doc.documentElement ().firstChild (); !group.isNull (); group = group.nextSibling ())
    {
      if (group.isElement () && group.nodeName () == "group")
	{
	  groupitem = new Q3ListViewItem (this, groupitem);
	  groupitem->setText (0, group.firstChild ().nodeValue ());
	  ///channel
	  for (QDomNode channelnode = group.firstChild (); !channelnode.isNull (); channelnode = channelnode.nextSibling ())
	    {
	      if (channelnode.isElement () && channelnode.nodeName () == "channel")
		{
		  ///element
		  item = new MyListItem (groupitem, item);
		  for (QDomNode namenode = channelnode.firstChild (); !namenode.isNull (); namenode = namenode.nextSibling ())
		    {
		      for (int i = 0; i < N_COLUMNS; i++)
			{
			  if (namenode.nodeName () == "sop_address")
			    item->setText (URL_COLUMN, namenode.firstChild ().firstChild ().nodeValue ());
			  else if (namenode.nodeName () == node_name[i])
			    item->setText (i, namenode.firstChild ().nodeValue ());
			}
		    }
		  ///filtering out channel
		  if (channelnode.toElement ().attribute ("type").toInt () == 1)
		    item->state |= 2;
		  ///disable null channel
		  else if (item->text (URL_COLUMN) == "null")
		    item->state |= 1;
		  else
		    {
		      static const int num[] = { USER_COLUMN, VISIT_COLUMN, QS_COLUMN,
			QC_COLUMN, KBPS_COLUMN
		      };
		      for (int i = 0; i < 5; i++)
			Max[num[i]] = item->text (num[i]).toInt () > Max[num[i]] ? item->text (num[i]).toInt () : Max[num[i]];

		    }
		}
	    }
	  ///group
	}
    }

  ///toggle null channel
  toggleNullChannel ();

  ////switch color
  switchChannelColor (static_cast < MainWindow * >(qApp->mainWidget ())->menubar->color_column);

  ///set open
  Q3ListViewItem *group = firstChild ();
  int minNum = int (childCount ()) < int (stateopen.count ())? int (childCount ()) : int (stateopen.count ());
  for (int i = 0; i < minNum; ++i)
    {
      if (*(stateopen.at (i)) == 1)
	group->setOpen (TRUE);
      group = group->nextSibling ();
    }

}



void
Channel::onRightButtonClicked (Q3ListViewItem * item, const QPoint & pos, int)
{
  if (item)
    if (item->depth () == 1)
      {
	popup->move (pos);
	popup->
	  setText
	  ("<table border=0 cellspacing=0 cellpadding=0><tr><td>"
	   + item->text (NAME_COLUMN) +
	   "</td></tr></table><hr>"
	   "<table border=0 cellspacing=0 cellpadding=0>" "<tr><td>Users: " + item->text (USER_COLUMN) + "</td><td>Visitors: " + item->text (VISIT_COLUMN) + "</td></tr>" "<tr><td>QualityC: " + item->text (QC_COLUMN) + "</td><td>QualityS: " + item->text (6) + "</td></tr>" "<tr><td>MediaType: " + item->text (STREAM_COLUMN) + "</td><td>KBps: " + item->text (KBPS_COLUMN) + "</td></tr>" "<tr><td>StartFrom: " + item->text (START_COLUMN) + "</td></tr></table><br>" + item->text (DESCRIPTION_COLUMN));
	popup->adjustSize ();
	popup->show ();
      }
}

void
Channel::onMouseMoved (Q3ListViewItem *)
{
  popup->hide ();
}


void
Channel::mouseMoveEvent (QMouseEvent * e)
{
  QPoint p (e->pos ());
  QRect pr (rect ());
  pr.setWidth (pr.width () - verticalScrollBar ()->width () * verticalScrollBar ()->isVisible ());
  pr.setHeight (pr.height () - horizontalScrollBar ()->height () * horizontalScrollBar ()->isVisible ());
  if (!pr.contains (p))
    {
      popup->hide ();
    }
}


void
Channel::toggleChannelSort ()
{
  if (!static_cast < MainWindow * >(qApp->mainWidget ())->menubar->menu_config->isItemChecked (CHANNEL_SORT))
    setSortColumn (-1);
  else
    setSortColumn (0);
}


void
Channel::toggleNullChannel ()
{
  bool t = static_cast < MainWindow * >(qApp->mainWidget ())->menubar->menu_config->isItemChecked (NULL_CHANNEL);

  Q3ListViewItem *group = firstChild ();
  while (group)
    {
      Q3ListViewItem *child = group->firstChild ();
      int count = 0;
      while (child)
	{
	  switch (((MyListItem *) child)->state)
	    {
	    case 0:
	      count++;
	      break;
	    case 1:
	      child->setEnabled (FALSE);
	      if (t)
		{
		  child->setVisible (TRUE);
		  count++;
		}
	      else
		child->setVisible (FALSE);
	      break;
	    case 2:
	    case 3:
	      child->setEnabled (FALSE);
	      child->setVisible (FALSE);
	    }
	  child = child->nextSibling ();
	}
      if (count == 0)
	group->setVisible (FALSE);
      else
	group->setVisible (TRUE);
      ///iterate to next group
      group = group->nextSibling ();
    }
}


void
Channel::switchChannelColor (int column)
{
  Q3ListViewItem *group = firstChild ();
  while (group)
    {
      Q3ListViewItem *child = group->firstChild ();
      while (child)
	{
	  if (child->isEnabled ())
	    {
	      int level = int (child->text (column).toInt () * (255. / Max[column]));

	      level = level < 0 ? 0 : level;
	      level = level > 255 ? 255 : level;

	      ((MyListItem *) child)->fcolor.setHsv (255 - level, 255, 125);
	      child->repaint ();
	    }
	  child = child->nextSibling ();
	}
      group = group->nextSibling ();
    }
}


///-------------------------------------------------------
void
Channel::onListItemClicked (Q3ListViewItem * item)
{
  if (item->depth () == 1)
    {
//      && tabwidget->tabToolTip(tabwidget->currentPage()) == "play") {

      //if the url is already played
      for (int i = 1; i < static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->count (); i++)
	{
	  if (static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->tabToolTip  (static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->widget (i)) == "vlc")
	    {
	      if (static_cast < PagePlay * >(static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->widget (i))->vlcfork->channelurl == item->text (URL_COLUMN))
		return;
	    }
	  else
	    {
	      if (static_cast < PagePlay * >(static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->widget (i))->mplayerfork->channelurl == item->text (URL_COLUMN))
		return;
	    }
	}

//add playtab and set it as current tab    
      static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->playTab ();

//      static_cast < PagePlay * >(static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->currentPage ())->fork->killSop ();
//      static_cast < PagePlay * >(static_cast<MainWindow*>(qApp->mainWidget())->tabwidget->currentPage ())
//       ->fork->killPlayer ();

      if (static_cast < MainWindow * >(qApp->mainWidget ())->menubar->player_column == VLC)
	{
	  if (static_cast < PagePlay * >(static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->currentPage ())->vlcfork->forkSop (item->text (URL_COLUMN), item->text (NAME_COLUMN), item->text (STREAM_COLUMN)) == TRUE)
	    {
	      ///add to history
	      static_cast < MainWindow * >(qApp->mainWidget ())->menubar->addToPopupMenu (static_cast < MainWindow * >(qApp->mainWidget ())->menubar->history, item->text (URL_COLUMN), item->text (NAME_COLUMN), item->text (STREAM_COLUMN));

	    }
	  else
	    fprintf (stderr, "failed to launch sp-sc!\n");
	}
      else
	{
	  if (static_cast < PagePlay * >(static_cast < MainWindow * >(qApp->mainWidget ())->tabwidget->currentPage ())->mplayerfork->forkSop (item->text (URL_COLUMN), item->text (NAME_COLUMN), item->text (STREAM_COLUMN)) == TRUE)
	    {
	      ///add to history
	      static_cast < MainWindow * >(qApp->mainWidget ())->menubar->addToPopupMenu (static_cast < MainWindow * >(qApp->mainWidget ())->menubar->history, item->text (URL_COLUMN), item->text (NAME_COLUMN), item->text (STREAM_COLUMN));

	    }
	  else
	    fprintf (stderr, "failed to launch sp-sc!\n");

	}

    }
}


void
Channel::onButtonSearchClicked ()
{
  if (currentItem () == 0 || pagechannel->editsearch->text ().isEmpty ())
    return;

  bool flagfirst = TRUE;
  Q3ListViewItem *item, *childitem, *focusitem = 0;
  ////assign initial value
  if (currentItem ()->depth () == 0)
    {
      item = currentItem ();
      childitem = item->firstChild ();
    }
  else
    {
      childitem = currentItem ();
      item = childitem->parent ();
      childitem = childitem->nextSibling ();
    }
  /////////find focus
  while (1)
    {
      while (childitem)
	{
	  if (childitem->text (NAME_COLUMN).contains (pagechannel->editsearch->text (), false) && childitem->isEnabled ())
	    {
	      setCurrentItem (childitem);
	      focusitem = childitem;
	      flagfirst = FALSE;
	      goto EXIT;
	    }
	  childitem = childitem->nextSibling ();
	}
      item = item->nextSibling ();
      if (!item)
	break;
      childitem = item->firstChild ();
    }
EXIT:;
  ////set color and find first focus if having failed finding focus so far
  item = firstChild ();
  while (item)
    {
      bool flag = FALSE;
      childitem = item->firstChild ();
      while (childitem)
	{
	  if (childitem->text (NAME_COLUMN).contains (pagechannel->editsearch->text (), false) && childitem->isEnabled ())
	    {
	      static_cast < MyListItem * >(childitem)->bcolor = Qt::yellow;
	      childitem->repaint ();
	      flag = TRUE;
	      if (flagfirst == TRUE)
		{
		  setCurrentItem (childitem);
		  focusitem = childitem;
		  flagfirst = FALSE;
		}
	    }
	  else
	    {
	      static_cast < MyListItem * >(childitem)->bcolor = Qt::white;
	      childitem->repaint ();
	    }
	  childitem = childitem->nextSibling ();
	}

      if (flag)
	{
	  item->setOpen (TRUE);
	}
      else
	{
	  item->setOpen (FALSE);
	}

      item = item->nextSibling ();
    }
//////////////////scroll to the first found item
  if (focusitem == 0)
    return;

  int value = itemPos (focusitem) - height () / 2;
  verticalScrollBar ()->setValue (value);
}


void
Channel::onButtonExpandClicked ()
{
  ///test if there are any closed items
  bool flagClose = false;
  Q3ListViewItem *item = firstChild ();
  while (item)
    {
      if (!item->isOpen ())
	{
	  flagClose = true;
	  break;
	}
      item = item->nextSibling ();
    }
  ///set open or close according to flagClose
  item = firstChild ();
  while (item)
    {
      if (flagClose)
	item->setOpen (TRUE);
      else
	item->setOpen (FALSE);
      item = item->nextSibling ();
    }
}


///---------------------------------------------
void
Channel::updateOpenState ()
{
  int minNum = int (childCount ()) < int (stateopen.count ())? int (childCount ()) : int (stateopen.count ());
  Q3ListViewItem *item = firstChild ();

  for (int i = 0; i < minNum; ++i)
    {
      if (item->isOpen ())
	*(stateopen.at (i)) = 1;
      else
	*(stateopen.at (i)) = 0;

      item = item->nextSibling ();
    }
  ///if childCount > stateopen.count
  if (minNum < childCount ())
    {
      while (item)
	{
	  if (item->isOpen ())
	    stateopen.append (1);
	  else
	    stateopen.append (0);

	  item = item->nextSibling ();
	}
    }
}
